* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
  OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=120;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
  TITLE; ODS TRACE OFF; ODS LISTING; ODS GRAPHICS;

***********************************************************************************;
*******             BEGIN MANIPULATION FOR HOFFMAN CHAPTER 2 DATA           *******;
*******                  CHANGE "filesave" to your directory                *******;
***********************************************************************************;

* Defining global variable for file location to be replaced in code below;
* \\Client\ precedes path in Virtual Desktop outside H drive;
  %LET filesave= C:\Dropbox\21SP_PSQF6242\PSQF6242_Example6;
* Using UIowa virtual desktop instead;
*%LET filesave= \\Client\C:\Dropbox\21SP_PSQF6242\PSQF6242_Example6;
* Location for SAS files for these models (uses macro variable filesave);
  LIBNAME filesave "&filesave.";

* Import chapter 2 example data into work library as Example6;
DATA work.Example6; SET filesave.SAS_Chapter2;
* Center quantitative predictors;
  age85 = age - 85;
  grip9 = grip - 9;
* Create dummy-coded binary predictors for dementia groups;
  demNF=.; demNC=.; * Create two new empty variables;
  IF demgroup=1 THEN DO; demNF=0; demNC=0; END; * Replace each for none group;
  IF demgroup=2 THEN DO; demNF=1; demNC=0; END; * Replace each for future group;
  IF demgroup=3 THEN DO; demNF=0; demNC=1; END; * Replace each for current group;
* Label new variables  note semi-colon is only at the end of ALL labels;
  LABEL   
  age85=     "age85: Age in Years (0=85)"
  grip9=     "grip9: Grip Strength in Pounds (0=9)"
  sexMW=     "sexMW: Sex (0=M, 1=W)"
  demNF=     "demNF: Dementia Contrast for None=0 vs Future=1"
  demNC=     "demNC: Dementia Contrast for None=0 vs Current=1"
  cognition= "cognition: Cognition Outcome"
  demgroup=  "demgroup: Dementia Group 1N 2F 3C";
* Select cases complete on variables;
  IF NMISS(cognition,age,grip,sexmw,demgroup)>0 THEN DELETE;
RUN;


***********************************************************************************;
*******                   BEGIN GENERAL LINEAR MODELS                       *******;
***********************************************************************************;

* Open output directory to save results to;
ODS RTF FILE="&filesave.\Example6_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

****** CREATING PREDICTED OUTCOMES OPTION 2 *******;

TITLE1 "SAS Combined Main Effects Only Model Predicting Cognition";
TITLE2 "Demonstrating how to get predicted outcomes using ESTIMATE statements";
PROC GLM DATA=work.Example6 NAMELEN=100;
     MODEL cognition = age85 grip9 sexMW demNF demNC / ALPHA=.05 CLPARM SOLUTION SS3 EFFECTSIZE;
     * We are ignoring the effects we would normally request for dementia for now;
     * Pred cognition outcomes holding sexMW=men, demNF=none, and demNC=none;
     ESTIMATE "Yhat for Age=80 Grip=6"  intercept 1 age85 -5 grip9 -3 sexMW 0 demNF 0 demNC 0;
     ESTIMATE "Yhat for Age=80 Grip=9"  intercept 1 age85 -5 grip9  0 sexMW 0 demNF 0 demNC 0;
     ESTIMATE "Yhat for Age=80 Grip=12" intercept 1 age85 -5 grip9  3 sexMW 0 demNF 0 demNC 0;
     ESTIMATE "Yhat for Age=85 Grip=6"  intercept 1 age85  0 grip9 -3 sexMW 0 demNF 0 demNC 0;
     ESTIMATE "Yhat for Age=85 Grip=9"  intercept 1 age85  0 grip9  0 sexMW 0 demNF 0 demNC 0;
     ESTIMATE "Yhat for Age=85 Grip=12" intercept 1 age85  0 grip9  3 sexMW 0 demNF 0 demNC 0;
     ESTIMATE "Yhat for Age=90 Grip=6"  intercept 1 age85  5 grip9 -3 sexMW 0 demNF 0 demNC 0;
     ESTIMATE "Yhat for Age=90 Grip=9"  intercept 1 age85  5 grip9  0 sexMW 0 demNF 0 demNC 0;
     ESTIMATE "Yhat for Age=90 Grip=12" intercept 1 age85  5 grip9  3 sexMW 0 demNF 0 demNC 0;
     ODS OUTPUT Estimates=work.EstMainEffects; * Save ESTIMATEs to dataset for plotting;
RUN; QUIT; TITLE1; TITLE2;

* Labeling saved ESTIMATES for use in plot;
* INDEX finds value in parentheses for that column;
DATA work.EstMainEffects; SET work.EstMainEffects;
     IF INDEX(Parameter,"Age=80")>0  THEN age=80;
     IF INDEX(Parameter,"Age=85")>0  THEN age=85;
     IF INDEX(Parameter,"Age=90")>0  THEN age=90;
     IF INDEX(Parameter,"Grip=6")>0  THEN grip=6;
     IF INDEX(Parameter,"Grip=9")>0  THEN grip=9;
     IF INDEX(Parameter,"Grip=12")>0 THEN grip=12;
RUN;

* Plot ESTIMATES -- grip as X by age;
PROC SGPLOT DATA=work.EstMainEffects;
     SERIES x=grip y=Estimate / GROUP=age; 
     XAXIS GRID LABEL="Grip Strength" VALUES=(6 TO 12 BY 3);
     YAXIS GRID LABEL="Predicted Cognition" VALUES=(15 TO 45 BY 5);
RUN; QUIT;


****** CREATING PREDICTED OUTCOMES OPTION 3 *******;

* Demonstrating how to get predicted outcomes using "fake people";
* Each row is a fake person for which to create a predicted outcome;
DATA work.FakePeople; INPUT PersonID age grip sexMW demNF demNC; 
* Center predictors; 
  age85=age-85; grip9=grip-9;
* Enter data;
  DATALINES; 
-99 80  6  0  0  0
-99 80  9  0  0  0 
-99 80 12  0  0  0
-99 85  6  0  0  0 
-99 85  9  0  0  0 
-99 85 12  0  0  0 
-99 90  6  0  0  0 
-99 90  9  0  0  0 
-99 90 12  0  0  0 
; RUN;
* Merge with real data;
DATA work.Example6; SET work.FakePeople work.Example6; RUN;

TITLE1 "SAS Combined Main Effects Only Model Predicting Cognition";
TITLE2 "Using dataset with fake people to get predicted outcomes as saved variable";
PROC GLM DATA=work.Example6 NAMELEN=100;
     MODEL cognition = age85 grip9 sexMW demNF demNC / ALPHA=.05 CLPARM SOLUTION SS3 EFFECTSIZE;
     * We are ignoring the effects we would normally request for dementia for now;
     * Request columns of predicted outcome and SE for all cases;
       OUTPUT OUT=work.PredOutcomes PREDICTED=Yhat STDP=SEyhat;
RUN; QUIT; TITLE1; TITLE2;

* Plot saved predicted values for fake people -- grip as X by age;
PROC SGPLOT DATA=work.PredOutcomes;
     WHERE PersonID=-99; * Only for fake people;
     SERIES x=grip y=Yhat / GROUP=age;
     XAXIS GRID LABEL="Grip Strength" VALUES=(6 TO 12 BY 3);
     YAXIS GRID LABEL="Predicted Cognition" VALUES=(15 TO 45 BY 5);
RUN; QUIT;


****** MODELS WITH SEX*AGE AND SEX*GRIP INTERACTION SLOPES *******;

TITLE1 "SAS GLM with 2 Interactions (Sex by Age, Sex by Grip)";
PROC GLM DATA=work.Example6 NAMELEN=100;
  MODEL cognition = age85 grip9 sexMW demNF demNC 
                    sexMW*age85 sexMW*grip9 / ALPHA=.05 CLPARM SOLUTION SS3 EFFECTSIZE;
  ESTIMATE "Mean Diff: Fut vs. Cur"       demNF -1 demNC 1; * B5-B4;
  CONTRAST "DFnum=2 F-test for Demgroup"  demNF 1, demNC 1;
  CONTRAST "DFnum=2 F-test for new interactions"  sexMW*age85 1, sexMW*grip9 1;
  * Simple slopes of age by sex;
  ESTIMATE "Age Slope for Men"    age85 1 sexMW*age85 0;
  ESTIMATE "Age Slope for Women"  age85 1 sexMW*age85 1;
   * Simple slopes of grip by sex;
  ESTIMATE "Grip Slope for Men"   grip9 1 sexMW*grip9 0;
  ESTIMATE "Grip Slope for Women" grip9 1 sexMW*grip9 1;
  * If you are NOT using fake people, you have to write these to create predicted outcomes;
  * Pred cognition outcomes holding demNF=none and demNC=none;
  ESTIMATE "Yhat: Men Age=80 Grip=6"    intercept 1 sexMW 0 age85 -5 grip9 -3 sexMW*age85  0 sexMW*grip9  0;
  ESTIMATE "Yhat: Men Age=80 Grip=12"   intercept 1 sexMW 0 age85 -5 grip9  3 sexMW*age85  0 sexMW*grip9  0;
  ESTIMATE "Yhat: Men Age=90 Grip=6"    intercept 1 sexMW 0 age85  5 grip9 -3 sexMW*age85  0 sexMW*grip9  0;
  ESTIMATE "Yhat: Men Age=90 Grip=12"   intercept 1 sexMW 0 age85  5 grip9  3 sexMW*age85  0 sexMW*grip9  0;
  ESTIMATE "Yhat: Women Age=80 Grip=6"  intercept 1 sexMW 1 age85 -5 grip9 -3 sexMW*age85 -5 sexMW*grip9 -3;
  ESTIMATE "Yhat: Women Age=80 Grip=12" intercept 1 sexMW 1 age85 -5 grip9  3 sexMW*age85 -5 sexMW*grip9  3;
  ESTIMATE "Yhat: Women Age=90 Grip=6"  intercept 1 sexMW 1 age85  5 grip9 -3 sexMW*age85  5 sexMW*grip9 -3;
  ESTIMATE "Yhat: Women Age=90 Grip=12" intercept 1 sexMW 1 age85  5 grip9  3 sexMW*age85  5 sexMW*grip9  3;
  ODS OUTPUT Estimates=EstSexInteract; * Save estimates to dataset;
RUN; QUIT; TITLE1; TITLE2;

* Labeling saved ESTIMATES for use in plot;
* INDEX finds value in parentheses for that column;
DATA work.EstSexInteract; LENGTH sex $6; SET work.EstSexInteract;
     IF INDEX(Parameter,"Men")>0     THEN sex="Men";
     IF INDEX(Parameter,"Women")>0   THEN sex="Women";
     IF INDEX(Parameter,"Age=80")>0  THEN age=80;
     IF INDEX(Parameter,"Age=90")>0  THEN age=90;
     IF INDEX(Parameter,"Grip=6")>0  THEN grip=6;
     IF INDEX(Parameter,"Grip=12")>0 THEN grip=12;
RUN;

* Plot ESTIMATES -- grip as X by sex holding age=80;
PROC SGPLOT DATA=work.EstSexInteract; WHERE age=80;
     SERIES x=grip y=Estimate / GROUP=sex; 
     XAXIS GRID LABEL="Grip Strength" VALUES=(6 TO 12 BY 3);
     YAXIS GRID LABEL="Predicted Cognition" VALUES=(15 TO 45 BY 5);
RUN; QUIT;

* Plot ESTIMATES -- age as X by sex holding grip=6;
PROC SGPLOT DATA=work.EstSexInteract; WHERE grip=6;
     SERIES x=age y=Estimate / GROUP=sex; 
     XAXIS GRID LABEL="Years of Age" VALUES=(80 TO 90 BY 5);
     YAXIS GRID LABEL="Predicted Cognition" VALUES=(15 TO 45 BY 5);
RUN; QUIT;


****** MODELS WITH AGE*GRIP INTERACTION SLOPE *******;

TITLE1 "SAS Eq 2.9: GLM with Age by Grip Interaction";
TITLE2 "Using dataset with fake people to get predicted outcomes as saved variable";
* Estimate model on data with fake people to make predictions;
PROC GLM DATA=work.Example6 NAMELEN=100;
  MODEL cognition = age85 grip9 sexMW demNF demNC age85*grip9 / ALPHA=.05 CLPARM SOLUTION SS3 EFFECTSIZE;
  ESTIMATE "Mean Diff: Fut vs. Cur"       demNF -1 demNC 1; * B5-B4;
  CONTRAST "DFnum=2 F-test for Demgroup"  demNF 1, demNC 1;
  CONTRAST "DFnum=3 F-test for age, grip, age*grip"  age85 1, grip9 1, age85*grip9 1;
  * Request columns of predicted outcome and SE for all cases;
  OUTPUT OUT=work.PredOutcomes PREDICTED=Yhat STDP=SEyhat;
  * Simple slopes of each main effect;
  ESTIMATE "Age Slope at Grip =  6" age85 1 age85*grip9 -3;
  ESTIMATE "Age Slope at Grip =  9" age85 1 age85*grip9  0;
  ESTIMATE "Age Slope at Grip = 12" age85 1 age85*grip9  3;
  ESTIMATE "Grip Slope at Age = 80" grip9 1 age85*grip9 -5;
  ESTIMATE "Grip Slope at Age = 85" grip9 1 age85*grip9  0;
  ESTIMATE "Grip Slope at Age = 90" grip9 1 age85*grip9  5; 
  * If you are NOT using fake people, you have to write these to create predicted outcomes;
  * Pred cognition outcomes holding sexMW=men, demNF=none, and demNC=none;
  ESTIMATE "Yhat for Age=80 Grip=6"  intercept 1 age85 -5 grip9 -3 age85*grip9  15;
  ESTIMATE "Yhat for Age=80 Grip=9"  intercept 1 age85 -5 grip9  0 age85*grip9   0;
  ESTIMATE "Yhat for Age=80 Grip=12" intercept 1 age85 -5 grip9  3 age85*grip9 -15;
  ESTIMATE "Yhat for Age=85 Grip=6"  intercept 1 age85  0 grip9 -3 age85*grip9   0;
  ESTIMATE "Yhat for Age=85 Grip=9"  intercept 1 age85  0 grip9  0 age85*grip9   0;
  ESTIMATE "Yhat for Age=85 Grip=12" intercept 1 age85  0 grip9  3 age85*grip9   0;
  ESTIMATE "Yhat for Age=90 Grip=6"  intercept 1 age85  5 grip9 -3 age85*grip9 -15;
  ESTIMATE "Yhat for Age=90 Grip=9"  intercept 1 age85  5 grip9  0 age85*grip9   0;
  ESTIMATE "Yhat for Age=90 Grip=12" intercept 1 age85  5 grip9  3 age85*grip9  15;
  ODS OUTPUT Estimates=EstAgebyGrip; * Save estimates to dataset;
RUN; QUIT; TITLE1; TITLE2;

* Plot saved predicted values for fake people -- age as X;
PROC SGPLOT DATA=work.PredOutcomes;
     WHERE PersonID=-99; * Only for fake people;
     SERIES x=age y=Yhat / GROUP=grip;
     XAXIS GRID LABEL="Years of Age" VALUES=(80 TO 90 BY 5);
     YAXIS GRID LABEL="Predicted Cognition" VALUES=(15 TO 45 BY 5);
RUN; QUIT;

* Plot saved predicted values for fake people -- grip as X;
PROC SGPLOT DATA=work.PredOutcomes;
     WHERE PersonID=-99; * Only for fake people;
     SERIES x=grip y=Yhat / GROUP=age;
     XAXIS GRID LABEL="Grip Strength" VALUES=(6 TO 12 BY 3);
     YAXIS GRID LABEL="Predicted Cognition" VALUES=(15 TO 45 BY 5);
RUN; QUIT;


TITLE1 "SAS Eq 2.9: MIXED with Age by Grip Interaction to Get COVB";
TITLE2 "Using dataset with fake people to get predicted outcomes as saved variable";
* Estimate model on data with fake people to make predictions;
PROC MIXED DATA=work.Example6 COVTEST NOCLPRINT NAMELEN=100 METHOD=REML;
  MODEL cognition = age85 grip9 sexMW demNF demNC age85*grip9
                     / SOLUTION DDFM=BW COVB OUTPM=PredOutcomes;
  * Saving info for regions to datasets: fixed effects and COVB;
  ODS OUTPUT SolutionF=FixAgeGrip COVB=CovBAgeGrip; 
RUN; TITLE1; TITLE2;
* Get correlation of yhat with y to compute R2;
PROC CORR NOSIMPLE DATA=work.PredOutcomes; VAR cognition; WITH pred; RUN; 


********************** CHANGE NOTHING IN THIS SECTION, JUST RUN IT **************;
* To use Regions Macro, enter;
* FixData =   Name of ODS SolutionF table that stores fixed effects for model;
* CovBData =  Name of ODS CovB table that stores XTX inv matrix for model;
* Pred =      Case-sensitive name of predictor effect regions are for; 
* Mod =       Case-sensitive name of moderator effect (for region values);
* ModCenter = Centering point of moderator predictor;
* Interact =  Case-sensitive name of interaction effect; 
* Order =     Order of entry of interaction in MODEL statement;
%MACRO Regions(FixData=,CovBData=,Pred=,Mod=,ModCenter=,Interact=,Order=);
DATA _NULL_; SET &FixData.; WHERE Effect="&Pred.";
     CALL SYMPUT('Bpred', Estimate); 
     CALL SYMPUT('SEpred', StdErr); RUN; 
DATA _NULL_; SET &FixData.; WHERE Effect="&Interact.";
     CALL SYMPUT('Binter', Estimate); 
     CALL SYMPUT('SEinter', StdErr); RUN; 
%LET order=%EVAL(&order.+1);
DATA _NULL_; SET &CovBData.; 
     WHERE INDEX(Effect,"&Pred.")>0 AND INDEX(Effect,"*")=0;
     CALL SYMPUT('CovPredInt', ROUND(Col&order.,.0001)); RUN;    
%PUT Bpred=&Bpred. SEpred=&SEpred. Binter=&Binter. 
     SEinter=&SEinter. CovPredInt=&CovPredInt.;
DATA Regions;
     A=(1.96*1.96)*(&SEinter.*&SEinter.)-(&Binter.*&Binter.);
     B=2*((1.96*1.96)*&CovPredInt.-(&Bpred.*&Binter.));
     C=(1.96*1.96*&SEpred.*&SEpred.)-(&Bpred.*&Bpred.);
     CenteredLower=((-1*B)+SQRT((B*B)-4*A*C))/(2*A); 
        CALL SYMPUT('cenlower',ROUND(CenteredLower,.001));
     CenteredUpper=((-1*B)-SQRT((B*B)-4*A*C))/(2*A); 
        CALL SYMPUT('cenupper',ROUND(CenteredUpper,.001));
     UncenteredLower=CenteredLower+&ModCenter.; 
        CALL SYMPUT('uncenlower',ROUND(UncenteredLower,.001));
     UncenteredUpper=CenteredUpper+&ModCenter.; 
        CALL SYMPUT('uncenupper',ROUND(UncenteredUpper,.001));
RUN;
TITLE7 "Regions of significance for &interact. interaction:";
TITLE8 "The &pred. slope will be significant at centered values of &mod. BELOW the lower bound"; 
TITLE9 "and ABOVE the upper bound, which translate to these uncentered lower and upper bounds."; 
PROC PRINT DATA=Regions NOOBS; VAR CenteredLower--UncenteredUpper; RUN; TITLE7; TITLE8; TITLE9;
%MEND Regions;
*********************************************************************************************

* Call macro for regions of significance for main effects of interaction;
%Regions(FixData=FixAgeGrip, CovBData=CovBAgeGrip, Pred=grip9, Mod=age85,
         ModCenter=85, Interact=age85*grip9, Order=6);
%Regions(FixData=FixAgeGrip, CovBData=CovBAgeGrip, Pred=age85, Mod=grip9,
         ModCenter=9, Interact=age85*grip9, Order=6);

* Close output;
ODS RTF CLOSE;

